package com.shanghaichengdi.downloadcenterserver.faced;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class AnalyseFaced {

  private final static String SHIP_COVER = "http://172.16.58.199:5000/shipCover";
  private final static String BOAT_FLAG = "http://172.16.58.199:3003/boat_flag";
  private final static String LIFE_JACKET = "http://172.16.58.199:5003/lifejacket";
  private final static String DOWNLOAD_FILE_DIR = "/shipAnalysis/HDImages/";
  private final static String SHIP_COVER_ANALYSE_DIR = "/disk/shipCover/";
  private final static String BOAT_FLAG_ANALYSE_DIR = "/disk/shareFlag/";
  private final static String LIFE_JACKET_ANALYSE_DIR = "/disk/shareLifejacket/";
  private final static String GH_CLOUD_SERVER = "http://10.83.68.108:8090/ghjgBayonetDataShareApi/analyse/receive";
  private final static String GH_CLOUD_HISTORY_SERVER = "http://10.83.68.108:8090/ghjgBayonetDataShareApi/analyse/history/receive";


  public static String shipCoverAnalyse(String id, String filePath, String bayonetId) {
    HttpRequest post = HttpUtil.createPost(SHIP_COVER);
    Map<String, String> headers = new HashMap<>();
    headers.put("Content-Type", "application/json;charset=utf-8");
    post.addHeaders(headers);
    Map<String, String> param = new LinkedHashMap<>();
    param.put("file", DOWNLOAD_FILE_DIR + filePath);
    String responseBody = post.body(JSONObject.toJSONString(param))
        .setReadTimeout(3600000)
        .execute()
        .body();
//    log.info("通用船舶货物识别完成,结果为:{}", responseBody);
    // 调用云上覆盖物/国旗接收接口
    JSONObject coverAnalyseResultData = JSONObject.parseObject(responseBody);
    JSONArray resultArray = (JSONArray) coverAnalyseResultData.get("result");
    JSONObject resultJsonObject = (JSONObject) resultArray.get(0);
    if (!Objects.isNull(resultJsonObject.get("class"))) {
      Map<String, Object> yunParam = new LinkedHashMap<>();
      yunParam.put("type", "覆盖物");
      yunParam.put("dir", SHIP_COVER_ANALYSE_DIR);
      yunParam.put("id", id);
      yunParam.put("data", responseBody);
      yunParam.put("bayonetId", bayonetId);
      String coverJsonString = JSONObject.toJSONString(yunParam);
//      log.info("覆盖物调用云上传参为:{}", coverJsonString);
      HttpRequest ghPost = HttpUtil.createPost(GH_CLOUD_SERVER);
      ghPost.addHeaders(headers);
      ghPost.body(coverJsonString)
          .setReadTimeout(3600000)
          .execute()
          .body();
    }
    return responseBody;
  }

  public static String shipCoverAnalyseHistory(String id, String filePath, String bayonetId,String passTime) {
    HttpRequest post = HttpUtil.createPost(SHIP_COVER);
    Map<String, String> headers = new HashMap<>();
    headers.put("Content-Type", "application/json;charset=utf-8");
    post.addHeaders(headers);
    Map<String, String> param = new LinkedHashMap<>();
    param.put("file", DOWNLOAD_FILE_DIR + filePath);
    String responseBody = post.body(JSONObject.toJSONString(param))
        .setReadTimeout(3600000)
        .execute()
        .body();
//    log.info("通用船舶货物识别完成,结果为:{}", responseBody);
    // 调用云上覆盖物/国旗接收接口
    JSONObject coverAnalyseResultData = JSONObject.parseObject(responseBody);
    JSONArray resultArray = (JSONArray) coverAnalyseResultData.get("result");
    JSONObject resultJsonObject = (JSONObject) resultArray.get(0);
    if (!Objects.isNull(resultJsonObject.get("class"))) {
      Map<String, Object> yunParam = new LinkedHashMap<>();
      yunParam.put("type", "覆盖物");
      yunParam.put("dir", SHIP_COVER_ANALYSE_DIR);
      yunParam.put("id", id);
      yunParam.put("passTime",passTime);
      yunParam.put("data", responseBody);
      yunParam.put("bayonetId", bayonetId);
      String coverJsonString = JSONObject.toJSONString(yunParam);
//      log.info("覆盖物调用云上传参为:{}", coverJsonString);
      HttpRequest ghPost = HttpUtil.createPost(GH_CLOUD_HISTORY_SERVER);
      ghPost.addHeaders(headers);
      ghPost.body(coverJsonString)
          .setReadTimeout(3600000)
          .execute()
          .body();
    }
    return responseBody;
  }

  public static String boatFlagAnalyse(String id, List<String> filePathList, String bayonetId) {
    String result = null;
    for (String filePath : filePathList) {
      HttpRequest post = HttpUtil.createPost(BOAT_FLAG);
      Map<String, String> headers = new HashMap<>();
      headers.put("Content-Type", "application/json;charset=utf-8");
      post.addHeaders(headers);
      Map<String, String> param = new LinkedHashMap<>();
      param.put("file", DOWNLOAD_FILE_DIR + filePath);
      String responseBody = post.body(JSONObject.toJSONString(param))
          .setReadTimeout(3600000)
          .execute()
          .body();
      JSONObject jsonObject = JSONObject.parseObject(responseBody);
      Boolean whetherHaveFlag = (Boolean) jsonObject.get("has_objects");
      if (whetherHaveFlag) {
//        log.info("当前编号:{}  船舶有国旗, 返回结果为:{}", id, responseBody);
        result = responseBody;
        break;
      } else {
        // 没有国旗的只保留全景图的分析结果
        String fullyImgResult = boatFlagAnalyse(id, filePathList.get(0));
//        log.warn("当前编号:{}  船舶没有国旗, 返回结果为:{}", id, responseBody);
        result = fullyImgResult;
      }
    }
//    log.info("国旗分析最终结果:{}", result);
    // 调用云上覆盖物/国旗接收接口
    Map<String, Object> yunParam = new LinkedHashMap<>();
    yunParam.put("type", "国旗");
    yunParam.put("dir", BOAT_FLAG_ANALYSE_DIR);
    yunParam.put("id", id);
    yunParam.put("data", result);
    yunParam.put("bayonetId", bayonetId);
    String flagJsonString = JSONObject.toJSONString(yunParam);
//    log.info("国旗调用云上传参为:{}", flagJsonString);
    HttpRequest ghPost = HttpUtil.createPost(GH_CLOUD_SERVER);
    Map<String, String> headers = new HashMap<>();
    headers.put("Content-Type", "application/json;charset=utf-8");
    ghPost.addHeaders(headers);
    ghPost.body(flagJsonString)
        .setReadTimeout(3600000)
        .execute()
        .body();
    return null;
  }

  public static String boatFlagAnalyseHistory(String id, List<String> filePathList, String bayonetId,String passTime) {
    String result = null;
    for (String filePath : filePathList) {
      HttpRequest post = HttpUtil.createPost(BOAT_FLAG);
      Map<String, String> headers = new HashMap<>();
      headers.put("Content-Type", "application/json;charset=utf-8");
      post.addHeaders(headers);
      Map<String, String> param = new LinkedHashMap<>();
      param.put("file", DOWNLOAD_FILE_DIR + filePath);
      String responseBody = post.body(JSONObject.toJSONString(param))
          .setReadTimeout(3600000)
          .execute()
          .body();
      JSONObject jsonObject = JSONObject.parseObject(responseBody);
      Boolean whetherHaveFlag = (Boolean) jsonObject.get("has_objects");
      if (whetherHaveFlag) {
        result = responseBody;
        break;
      } else {
        // 没有国旗的只保留全景图的分析结果
        String fullyImgResult = boatFlagAnalyse(id, filePathList.get(0));
        result = fullyImgResult;
      }
    }
    // 调用云上覆盖物/国旗接收接口
    Map<String, Object> yunParam = new LinkedHashMap<>();
    yunParam.put("type", "国旗");
    yunParam.put("dir", BOAT_FLAG_ANALYSE_DIR);
    yunParam.put("id", id);
    yunParam.put("passTime", passTime);
    yunParam.put("data", result);
    yunParam.put("bayonetId", bayonetId);
    String flagJsonString = JSONObject.toJSONString(yunParam);
    HttpRequest ghPost = HttpUtil.createPost(GH_CLOUD_HISTORY_SERVER);
    Map<String, String> headers = new HashMap<>();
    headers.put("Content-Type", "application/json;charset=utf-8");
    ghPost.addHeaders(headers);
    ghPost.body(flagJsonString)
        .setReadTimeout(3600000)
        .execute()
        .body();
    return null;
  }

  public static String boatFlagAnalyse(String id, String filePath) {
    HttpRequest post = HttpUtil.createPost(BOAT_FLAG);
    Map<String, String> headers = new HashMap<>();
    headers.put("Content-Type", "application/json;charset=utf-8");
    post.addHeaders(headers);
    Map<String, String> param = new LinkedHashMap<>();
    param.put("file", DOWNLOAD_FILE_DIR + filePath);
    return post.body(JSONObject.toJSONString(param))
        .setReadTimeout(3600000)
        .execute()
        .body();
  }

  public static String lifeJacketAnalyse(String id, List<String> filePathList, String bayonetId) {
    String result = null;
    for (String filePath : filePathList) {
      HttpRequest post = HttpUtil.createPost(LIFE_JACKET);
      Map<String, String> headers = new HashMap<>();
      headers.put("Content-Type", "application/json;charset=utf-8");
      post.addHeaders(headers);
      Map<String, String> param = new LinkedHashMap<>();
      param.put("file", DOWNLOAD_FILE_DIR + filePath);
      String responseBody = post.body(JSONObject.toJSONString(param))
              .setReadTimeout(3600000)
              .execute()
              .body();
      JSONObject jsonObject = JSONObject.parseObject(responseBody);
      List<Object> strings = JSONArray.parseArray(jsonObject.get("result").toString());
      JSONObject jsonObject1 = JSONObject.parseObject(strings.get(0).toString());
      if(String.valueOf(1).equals(jsonObject1.get("is_wear") == null ? null : jsonObject1.get("is_wear").toString())) {
        // 如果whetherWearLifejacket为1
        result = responseBody;
        break;
      } else {
        // 如果whetherWearLifejacket为0或空值
        // 处理穿救生衣或无人时的分析结果
        result = responseBody;
      }
    };
//    log.info("救生衣分析最终结果:{}", result);
    // 调用云上覆盖物/国旗/救生衣接收接口
    Map<String, Object> yunParam = new LinkedHashMap<>();
    yunParam.put("type", "救生衣");
    yunParam.put("dir", LIFE_JACKET_ANALYSE_DIR);
    yunParam.put("id", id);
    yunParam.put("data", result);
    yunParam.put("bayonetId", bayonetId);
    String lifejacketJsonString = JSONObject.toJSONString(yunParam);
//    log.info("救生衣调用云上传参为:{}", lifejacketJsonString);
    HttpRequest ghPost = HttpUtil.createPost(GH_CLOUD_SERVER);
    Map<String, String> headers = new HashMap<>();
    headers.put("Content-Type", "application/json;charset=utf-8");
    ghPost.addHeaders(headers);
    ghPost.body(lifejacketJsonString)
            .setReadTimeout(3600000)
            .execute()
            .body();
    return null;
  }
}
